/***********************************************************************/
/*  This file is part of the ARM Toolchain package                     */
/*  Copyright (c) 2010 Keil - An ARM Company. All rights reserved.     */
/***********************************************************************/
/*                                                                     */
/*  FlashDev.C:  Flash Programming Functions adapted                   */
/*               for New Device 256kB Flash                            */
/*                                                                     */
/***********************************************************************/

#include "..\FlashOS.H"        // FlashOS Structures

#include <stdint.h>
#include <stdio.h>

//#define WITH_SRVMODE

#define M8(adr)  (*((volatile unsigned char  *) (adr)))
#define M16(adr) (*((volatile unsigned short *) (adr)))
#define M32(adr) (*((volatile unsigned long  *) (adr)))
	
#define FLASH_MAGIC_KEY		0xA4420000
#define FLASH_WRITE			0x01
#define FLASH_PAGE_ERASE	0x02
#define FLASH_FULL_ERASE	0x04

//#ifdef __cplusplus
//  #define   __I     volatile             /*!< Defines 'read only' permissions                 */
//#else
  #define   __I     volatile const       /*!< Defines 'read only' permissions                 */
//#endif
#define     __O     volatile             /*!< Defines 'write only' permissions                */
#define     __IO    volatile             /*!< Defines 'read / write' permissions              */

/* ================================================================================ */
/* ================                  NT_BOOTFLASH                  ================ */
/* ================================================================================ */


/**
  * @brief 1.0 (NT_BOOTFLASH)
  */

typedef struct {                                    /*!< NT_BOOTFLASH Structure                                                */
 
    __IO uint32_t  FMA;                             /*!< Address bootflash register                                            */
    __IO uint32_t  FMD1;                            /*!< Data words bootable flash register 1                                  */
    __O  uint32_t  FMC;                             /*!< Command register                                                      */
    __I  uint32_t  FCIS;                            /*!< Status register                                                       */
    __IO uint32_t  FCIM;                            /*!< Interrupt mask register                                               */
    __IO uint32_t  FCIC;                            /*!< Command register                                                      */
  __I  uint32_t  RESERVED[14];
    __IO uint32_t  FMD2;                            /*!< Data words bootable flash register 2                                  */
    __IO uint32_t  FMD3;                            /*!< Data words bootable flash register 3                                  */
    __IO uint32_t  FMD4;                            /*!< Data words bootable flash register 4                                  */
  __I  uint32_t  RESERVED1[9];
    __IO uint32_t  T_ACC;                           /*!< Register the delay of setting addresses to read data from the				 */	
} NT_BOOTFLASH_Type;

#define NT_BOOTFLASH_BASE               0xA001C000UL
#define NT_BOOTFLASH                    ((NT_BOOTFLASH_Type       *) NT_BOOTFLASH_BASE)

#define FULL_FLASH_ERASE_ADDR	0x80030164
// Define for Read State H.2
#define READ_H2 ((M8(0x80017000+0x404) >> 2)&0x01)

/* 
   Mandatory Flash Programming Functions (Called by FlashOS):
                int Init        (unsigned long adr,   // Initialize Flash
                                 unsigned long clk,
                                 unsigned long fnc);
                int UnInit      (unsigned long fnc);  // De-initialize Flash
                int EraseSector (unsigned long adr);  // Erase Sector Function
                int ProgramPage (unsigned long adr,   // Program Page Function
                                 unsigned long sz,
                                 unsigned char *buf);

   Optional  Flash Programming Functions (Called by FlashOS):
                int BlankCheck  (unsigned long adr,   // Blank Check
                                 unsigned long sz,
                                 unsigned char pat);
                int EraseChip   (void);               // Erase complete Device
      unsigned long Verify      (unsigned long adr,   // Verify Function
                                 unsigned long sz,
                                 unsigned char *buf);

       - BlanckCheck  is necessary if Flash space is not mapped into CPU memory space
       - Verify       is necessary if Flash space is not mapped into CPU memory space
       - if EraseChip is not provided than EraseSector for all sectors is called
*/


/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {

  /* Add your Code */
	/* Configurate time acces to FLASH */
	NT_BOOTFLASH->T_ACC = 6;
  return (0);                                  // Finished without Errors
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

  /* Add your Code */
  return (0);                                  // Finished without Errors
}


/*
 *  Detected Service by read first 16 byte from 0x0000
 *    Return Value:   0 -  - OK (no Service Mode),  1 - Service Mode
 */

int ServiceModeDetect()
{
  unsigned char i;
	unsigned char flag=1;
	for (i=0;i<0x10;i+=4) if (M32((unsigned long)i)>0) flag = 0;
	return flag;
}	

/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseChip (void) {
  unsigned int timeout = 0xFFFFFFFF;
  unsigned char stat=0;
  if (ServiceModeDetect())
  {
  //Full Flash Erase in Service Mode
  M32(FULL_FLASH_ERASE_ADDR)=1;
    while ((M32(FULL_FLASH_ERASE_ADDR)<3) & timeout) timeout--;
    if (timeout >0) return (0);
      else return (1);
  } else  {
  //Flash Erase in Normal Mode 
  NT_BOOTFLASH->FMC = FLASH_MAGIC_KEY | FLASH_FULL_ERASE;
  while (stat == 0) stat = NT_BOOTFLASH->FCIS;
  if (stat == 0x2) return (1);  
  return (0);  			// Finished without Errors   
  }  

//  return (0);                                  // Finished without Errors
}

/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseSector (unsigned long adr) {
  /* Add your Code */
	unsigned char stat;	
	NT_BOOTFLASH->FMA = adr;
	NT_BOOTFLASH->FMC = FLASH_MAGIC_KEY | FLASH_PAGE_ERASE;
	stat = NT_BOOTFLASH->FCIS;
	while (stat == 0) {
		stat = NT_BOOTFLASH->FCIS;
	}	
	if (stat == 0x2) return (1);
	else return (0);                                  // Finished without Errors
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {

  /* Add your Code */
           int  i;
	unsigned char stat;
  for (i = 0; i < ((sz+15)/16); i++)  {
		NT_BOOTFLASH->FMA = adr;
		NT_BOOTFLASH->FCIC = 0x03;		
		NT_BOOTFLASH->FMD1 = *((unsigned long *) buf); 		// Write Data[31:0] from Buffer
		buf += 4;                                 			// Next Data
		NT_BOOTFLASH->FMD2 = *((unsigned long *) buf); 		// Write Data[63:32] from Buffer
		buf += 4;                                 			// Next Data
		NT_BOOTFLASH->FMD3 = *((unsigned long *) buf); 		// Write Data[95:64] from Buffer
		buf += 4;                                 			// Next Data
		NT_BOOTFLASH->FMD4 = *((unsigned long *) buf); 		// Write Data[127:96] from Buffer
		buf += 4;                                 			// Next Data			
		adr += 16;                                 			// Next Address
		NT_BOOTFLASH->FMC = FLASH_MAGIC_KEY | FLASH_WRITE; // WRITE DATA to BOOTFLASH
		stat = 0;
		while (stat == 0) stat = NT_BOOTFLASH->FCIS;
		if (stat == 0x2) return (1);
	}
  return (0);                                  // Finished without Errors
}